##
## 6.103 -- Vectors and Data Frames
##
## Martin Schedlbauer, PhD
## Northeastern University
##

# In R, a vector is similar to a list or array in other programming languages. 
# It is a collection of elements of the same basic type: 
# numeric, character, or logical (Boolean)
#
# Vectors cannot contain class objects, only built-in primitive types.
#
# As an aside, a list in R is a collection of mixed data types. 
# This tutorial applies to vectors only.
# 
# Vectors emerge in different ways in R:
# 
#   result of manually creating them
#   result of a call to a function
#   extraction of elements from a vector
#   column in a data frame
# 
# There are no specific packages required to manipulate, use, create, or 
# access vectors.

v.numeric <- c(23, 77, 12, 98, -23, 0)
v.char <- c('R', 'Java', 'Python', 'C++', 'LISP')
v.logical <- c(TRUE, F, FALSE, T, T, T)

v.mixed <- c(23, 0, 'R', TRUE)

print(v.mixed)

set.seed(98088)
v <- round(runif(50, min = 0, max = 10),0)
v

length(v)

v.a <- c(23, 33, 10, 8, 7)

r <- v.a * 2.5

print(r)

r[2]

r[23]

# Approach 1: Using Loops
v.a <- c(23, 33, 10, 8, 7)
v.b <- c(0.3, 0.5, 0.1, 0.9, 0.8)

n <- length(v.a)

r <- 0
for (i in 1:n) {
  r <- r + (v.a[i] * v.b[i])
}

print(paste0("Approach 1 (loop): ", r))

v.prod <- v.a * v.b
v.sum <- sum(v.prod)

print(paste0("Approach 2 (vector operation and function): ", v.sum))

dp <- v.a %*% v.b
print(paste0("Approach 3 (dot product operand): ", dp))

print(v)

# access a single element at position 3
v[3]

3:10

10:3

# access element 3 through 5
v[3:5]

# access the first element
v[1]

# access the last element
v[length(v)]

# access the next to last element
v[length(v)-1]

# access every other element
v[seq(from = 1, to = length(v), by = 2)]

# access specific elements at positions 1, 5, and 7
i <- c(1,5,7)
v[i]

length(v < 5)

v <- c(2,5,7,1)
v == 5
v = 5

v != 5
v[which(v != 5)]

any(v < 5)


##
## DATA FRAMES
##

mtcars
nrow(mtcars)

v <- mtcars[4,3]
x = mtcars[3,4]

print(paste0("v = ",v," and x = ",x))

mtcars[,3]
mtcars[5:10,]
mtcars[c(3,6,12),]
mtcars[c(3,6,12),c(1,2,4)]

mtcars[,1]
mtcars$mpg
mtcars[,"mpg"]
mtcars[,c("mpg","hp")]

mtcars$foo <- 99
head(mtcars)

colnames(mtcars)[1] <- "miPerGa"

df1 <- data.frame(state = c('Arizona','Georgia', 'New York','Indiana','Washington','Texas'),
                  code = as.factor(c('AZ','GA','NY','IN','WA','TX')),
                  score = c(62,47,55,74,31,85))

head(df1)

df1[nrow(df1),]
df1[nrow(df1)+1,3] <- 87

df1 <- NULL

rm(df)

